@@ -6,15 +6,17 @@ import os
 import time
 from concurrent import futures
 from copy import deepcopy
-from dataclasses import dataclass
 from pathlib import Path
 from typing import IO, Union, List
+from collections import defaultdict
+import re
 
 # Non standard libraries
 import pandas as pd
 import requests
 
 from aanalytics2 import config, connector, token_provider
+from .projects import *
 
 JsonOrDataFrameType = Union[pd.DataFrame, dict]
 JsonListOrDataFrameType = Union[pd.DataFrame, List[dict]]
@@ -225,177 +227,6 @@ class Login:
         return analytics
 
 
-@dataclass
-class Project:
-    """
-    This dataclass extract the information retrieved from the getProjet method.
-    It flatten the elements and gives you insights on what your project contains.
-    """
-
-    def __init__(self, projectDict: dict = None):
-        if projectDict is None:
-            raise Exception("require a dictionary")
-        self.id: str = projectDict.get('id', '')
-        self.name: str = projectDict.get('name', '')
-        self.description: str = projectDict.get('description', '')
-        self.rsid: str = projectDict.get('rsid', '')
-        self.ownerName: str = projectDict['owner'].get('name', '')
-        self.ownerId: int = projectDict['owner'].get('id', '')
-        self.ownerEmail: int = projectDict['owner'].get('login', '')
-        self.template: bool = projectDict.get('companyTemplate', False)
-        self.version: str = None
-        if 'definition' in projectDict.keys():
-            definition: dict = projectDict['definition']
-            self.version: str = definition['version']
-            self.curation: bool = definition.get('isCurated', False)
-            if definition.get('device', 'desktop') != 'cell':
-                infos = self._findPanelsInfos(definition['workspaces'][0])
-                self.nbPanels: int = infos["nb_Panels"]
-                self.nbSubPanels: int = 0
-                self.subPanelsTypes: list = []
-                for panel in infos["panels"]:
-                    self.nbSubPanels += infos["panels"][panel]['nb_subPanels']
-                    self.subPanelsTypes += infos["panels"][panel]['subPanels_types']
-                self.elementsUsed: dict = self._findElements(definition['workspaces'][0])
-                self.nbElementsUsed: int = len(self.elementsUsed['dimensions']) + len(
-                    self.elementsUsed['metrics']) + len(self.elementsUsed['segments']) + len(
-                    self.elementsUsed['calculatedMetrics'])
-
-    def _findPanelsInfos(self, workspace: dict = None) -> dict:
-        """
-        Return a dict of the different information for each Panel.
-        Arguments:
-            workspace : REQUIRED : the workspace dictionary. 
-        """
-        dict_data = {'workspace_id': workspace['id']}
-        dict_data['nb_Panels'] = len(workspace['panels'])
-        dict_data['panels'] = {}
-        for panel in workspace['panels']:
-            dict_data["panels"][panel['id']] = {}
-            dict_data["panels"][panel['id']]['name'] = panel.get('name', 'Default Name')
-            dict_data["panels"][panel['id']]['nb_subPanels'] = len(panel['subPanels'])
-            dict_data["panels"][panel['id']]['subPanels_types'] = [subPanel['reportlet']['type'] for subPanel in
-                                                                   panel['subPanels']]
-        return dict_data
-
-    def _findElements(self, workspace: dict) -> list:
-        """
-        Returns the list of dimensions used in the FreeformReportlet. 
-        Arguments :
-            workspace : REQUIRED : the workspace dictionary.
-        """
-        dict_elements: dict = {'dimensions': [], "metrics": [], 'segments': [], "reportSuites": [],
-                               'calculatedMetrics': []}
-        for panel in workspace['panels']:
-            if "reportSuite" in panel.keys():
-                dict_elements['reportSuites'].append(panel['reportSuite']['id'])
-            elif "rsid" in panel.keys():
-                dict_elements['reportSuites'].append(panel['rsid'])
-            filters: list = panel['segmentGroups']
-            if len(filters) > 0:
-                for element in filters:
-                    typeElement = element['componentOptions'][0]['component']['type']
-                    idElement = element['componentOptions'][0]['component']['id']
-                    if typeElement == "Segment":
-                        dict_elements['segments'].append(idElement)
-                    if typeElement == "DimensionItem":
-                        clean_id: str = idElement[:idElement.find(
-                            '::')]  ## cleaning this type of element : 'variables/evar7.6::3000623228'
-                        dict_elements['dimensions'].append(clean_id)
-            for subPanel in panel['subPanels']:
-                if subPanel['reportlet']['type'] == "FreeformReportlet":
-                    reportlet = subPanel['reportlet']
-                    rows = reportlet['freeformTable']
-                    if 'dimension' in rows.keys():
-                        dict_elements['dimensions'].append(rows['dimension']['id'])
-                    if len(rows["staticRows"]) > 0:
-                        for row in rows["staticRows"]:
-                            ## I have to get a temp dimension to clean them before loading them in order to avoid counting them multiple time for each rows.
-                            temp_list_dim = []
-                            componentType: str = row['component']['type']
-                            if componentType == "DimensionItem":
-                                temp_list_dim.append(row['component']['id'])
-                            elif componentType == "Segments":
-                                dict_elements['segments'].append(row['component']['id'])
-                            elif componentType == "Metric":
-                                dict_elements['metrics'].append(row['component']['id'])
-                            elif componentType == "CalculatedMetric":
-                                dict_elements['calculatedMetrics'].append(row['component']['id'])
-                        if len(temp_list_dim) > 0:
-                            temp_list_dim = list(set([el[:el.find('::')] for el in temp_list_dim]))
-                        for dim in temp_list_dim:
-                            dict_elements['dimensions'].append(dim)
-                    columns = reportlet['columnTree']
-                    for node in columns['nodes']:
-                        temp_data = self._recursiveColumn(node)
-                        dict_elements['calculatedMetrics'] += temp_data['calculatedMetrics']
-                        dict_elements['segments'] += temp_data['segments']
-                        dict_elements['metrics'] += temp_data['metrics']
-                        if len(temp_data['dimensions']) > 0:
-                            for dim in set(temp_data['dimensions']):
-                                dict_elements['dimensions'].append(dim)
-        dict_elements['metrics'] = list(set(dict_elements['metrics']))
-        dict_elements['segments'] = list(set(dict_elements['segments']))
-        dict_elements['dimensions'] = list(set(dict_elements['dimensions']))
-        dict_elements['calculatedMetrics'] = list(set(dict_elements['calculatedMetrics']))
-        return dict_elements
-
-    def _recursiveColumn(self, node: dict = None, temp_data: dict = None):
-        """
-        recursive function to fetch elements in column stack
-        """
-        if temp_data is None:
-            temp_data: dict = {'dimensions': [], "metrics": [], 'segments': [], "reportSuites": [],
-                               'calculatedMetrics': []}
-        componentType: str = node['component']['type']
-        if componentType == "Metric":
-            temp_data['metrics'].append(node['component']['id'])
-        elif componentType == "CalculatedMetric":
-            temp_data['calculatedMetrics'].append(node['component']['id'])
-        elif componentType == "Segments":
-            temp_data['segments'].append(node['component']['id'])
-        elif componentType == "DimensionItem":
-            old_id: str = node['component']['id']
-            new_id: str = old_id[:old_id.find('::')]
-            temp_data['dimensions'].append(new_id)
-        if len(node['nodes']) > 0:
-            for new_node in node['nodes']:
-                temp_data = self._recursiveColumn(new_node, temp_data=temp_data)
-        return temp_data
-
-    def to_dict(self) -> dict:
-        """
-        transform the class into a dictionary
-        """
-        obj = {
-            'id': self.id,
-            'name': self.name,
-            'description': self.description,
-            'rsid': self.rsid,
-            'ownerName': self.ownerName,
-            'ownerId': self.ownerId,
-            'ownerEmail': self.ownerEmail,
-            'template': self.template,
-        }
-        add_object = {}
-        if hasattr(self, 'nbPanels'):
-            add_object = {
-                'curation': self.curation,
-                'version': self.version,
-                'nbPanels': self.nbPanels,
-                'nbSubPanels': self.nbSubPanels,
-                'subPanelsTypes': self.subPanelsTypes,
-                'nbElementsUsed': self.nbElementsUsed,
-                'dimensions': self.elementsUsed['dimensions'],
-                'metrics': self.elementsUsed['metrics'],
-                'segments': self.elementsUsed['segments'],
-                'calculatedMetrics': self.elementsUsed['calculatedMetrics'],
-                'rsids': self.elementsUsed['reportSuites'],
-            }
-        full_obj = {**obj, **add_object}
-        return full_obj
-
-
 class Analytics:
     """
     Class that instantiate a connection to a single login company.
@@ -431,6 +262,10 @@ class Analytics:
         self.header['x-proxy-global-company-id'] = company_id
         self.endpoint_company = f"{self._endpoint}/{company_id}"
         self.company_id = company_id
+        self.listProjectIds = []
+        self.projectsDetails = {}
+        self.segments = []
+        self.calculatedMetrics = []
         try:
             import importlib.resources as pkg_resources
             pathLOGS = pkg_resources.path(
@@ -449,6 +284,24 @@ class Analytics:
         except:
             self.LOGS_EVENT_TYPE = "no data"
 
+    def __str__(self)->str:
+        obj = {
+            "endpoint" : self.endpoint_company,
+            "companyId" : self.company_id,
+            "header" : self.header,
+            "token" : self.connector.config['token']
+        }
+        return json.dumps(obj,indent=4)
+    
+    def __repr__(self)->str:
+        obj = {
+            "endpoint" : self.endpoint_company,
+            "companyId" : self.company_id,
+            "header" : self.header,
+            "token" : self.connector.config['token']
+        }
+        return json.dumps(obj,indent=4)
+
     def refreshToken(self, token: str = None):
         if token is None:
             raise AttributeError(
@@ -902,11 +755,12 @@ class Analytics:
                     f'Saving data in file : {os.getcwd()}{os.sep}segments.csv')
         return segments
 
-    def getSegment(self, segment_id: str = None, *args):
+    def getSegment(self, segment_id: str = None,full:bool=False, *args):
         """
         Get a specific segment from the ID. Returns the object of the segment.
         Arguments:
             segment_id : REQUIRED : the segment id to retrieve.
+            full : OPTIONAL : Add all possible options
         Possible args:
             - "reportSuiteName" : string : to retrieve reportSuite attached to the segment
             - "ownerFullName" : string : to retrieve ownerFullName attached to the segment
@@ -927,6 +781,8 @@ class Analytics:
             if element not in ValidArgs:
                 args.remove(element)
         params = {'expansion': ','.join(args)}
+        if full:
+            params = {'expansion': ','.join(ValidArgs)}
         res = self.connector.getData(self.endpoint_company + path, params=params, headers=self.header)
         return res
 
@@ -947,6 +803,21 @@ class Analytics:
             headers=self.header
         )
         return seg
+    
+    def createSegmentValidate(self, segmentJSON: dict = None) -> object:
+        """
+        Method that validate a new segment based on the dictionary passed to it.
+        Arguments:
+            segmentJSON : REQUIRED : the dictionary that represents the JSON statement for the segment.
+            More information at this address <https://adobedocs.github.io/analytics-2.0-apis/#/segments/segments_createSegment>
+        """
+        if segmentJSON is None:
+            print('No segment data has been pushed')
+            return None
+        data = deepcopy(segmentJSON)
+        path = "/segments/validate"
+        seg = self.connector.postData(self.endpoint_company +path,data=data)
+        return seg
 
     def updateSegment(self, segmentID: str = None, segmentJSON: dict = None) -> object:
         """
@@ -987,6 +858,7 @@ class Analytics:
             rsids_list: list = None,
             extended_info: bool = False,
             save=False,
+            format:str='df',
             **kwargs
     ) -> pd.DataFrame:
         """
@@ -1002,6 +874,7 @@ class Analytics:
             extended_info : OPTIONAL : additional segment metadata fields to include on response (list)
                 additional infos: reportSuiteName,definition, ownerFullName, modified, tags, compatibility
             save : OPTIONAL : If set to True, it will save the info in a csv file (Default False)
+            format : OPTIONAL : format of the output. 2 values "df" for dataframe and "raw" for raw json.
         Possible kwargs:
             limit : number of segments retrieved by request. default 500: Limited to 1000 by the AnalyticsAPI.(int)
         """
@@ -1035,14 +908,36 @@ class Analytics:
                                                  self._getCalcMetrics, params=params, headers=self.header)
                 data += metrics['content']
                 lastPage = metrics['lastPage']
+        if format == "raw":
+            if save:
+                with open('calculated_metrics.json','w') as f:
+                    f.write(json.dumps(data,indent=4))
+            return data
         df_calc_metrics = pd.DataFrame(data)
         if save:
             df_calc_metrics.to_csv('calculated_metrics.csv', sep='\t')
         return df_calc_metrics
 
-    def createCalculatedMetrics(self, metricJSON: dict = None) -> object:
+    def getCalculatedMetric(self,calculatedMetricId:str=None,full:bool=True)->dict:
         """
-        Method that create a specific calculated based on the dictionary passed to it.
+        Return a dictionary on the calculated metrics requested.
+        Arguments:
+            calculatedMetricId : REQUIRED : The calculated metric ID to be retrieved.
+            full : OPTIONAL : additional segment metadata fields to include on response (list)
+                additional infos: reportSuiteName,definition, ownerFullName, modified, tags, compatibility
+        """
+        if calculatedMetricId is None:
+            raise ValueError("Require a calculated metrics ID")
+        params = {}
+        if full:
+            params.update({'expansion': 'reportSuiteName,definition,ownerFullName,modified,tags,categories,compatibility'})
+        path = f"/calculatedmetrics/{calculatedMetricId}"
+        res = self.connector.getData(self.endpoint_company+path,params=params)
+        return res
+
+    def createCalculatedMetric(self, metricJSON: dict = None) -> dict:
+        """
+        Method that create a specific calculated metric based on the dictionary passed to it.
         Arguments:
             metricJSON : REQUIRED : Calculated Metrics information to create. (Required: name, definition, rsid)
             More information can be found at this address https://adobedocs.github.io/analytics-2.0-apis/#/calculatedmetrics/calculatedmetrics_createCalculatedMetric
@@ -1056,6 +951,23 @@ class Analytics:
         cm = self.connector.postData(self.endpoint_company +
                                      self._getCalcMetrics, headers=self.header, data=metricJSON)
         return cm
+    
+    def createCalculatedMetricValidate(self,metricJSON: dict=None)->dict:
+        """
+        Method that validate a specific calculated metrics definition based on the dictionary passed to it.
+        Arguments:
+            metricJSON : REQUIRED : Calculated Metrics information to create. (Required: name, definition, rsid)
+            More information can be found at this address https://adobedocs.github.io/analytics-2.0-apis/#/calculatedmetrics/calculatedmetrics_createCalculatedMetric
+        """
+        if metricJSON is None or type(metricJSON) != dict:
+            raise Exception(
+                "Expected a dictionary to create the calculated metrics")
+        if 'name' not in metricJSON.keys() or 'definition' not in metricJSON.keys() or 'rsid' not in metricJSON.keys():
+            raise KeyError(
+                'Expected "name", "definition" and "rsid" in the data')
+        path = "/calculatedmetrics/validate"
+        cm = self.connector.postData(self.endpoint_company+path, data=metricJSON)
+        return cm
 
     def updateCalculatedMetric(self, calcID: str = None, calcJSON: dict = None) -> object:
         """
@@ -1090,7 +1002,7 @@ class Analytics:
         )
         return cm
 
-    def getDateRanges(self, extended_info: bool = False, save: bool = False, includeType: str = 'all',
+    def getDateRanges(self, extended_info: bool = False, save: bool = False, includeType: str = 'all',verbose:bool=False,
                       **kwargs) -> pd.DataFrame:
         """
         Get the list of date ranges available for the user.
@@ -1113,7 +1025,8 @@ class Analytics:
         dateRanges = self.connector.getData(
             self.endpoint_company + self._getDateRanges,
             params=params,
-            headers=self.header
+            headers=self.header,
+            verbose=verbose
         )
         data = dateRanges['content']
         df_dates = pd.DataFrame(data)
@@ -1355,6 +1268,7 @@ class Analytics:
         if limit is not None:
             params['limit'] = limit
         res = self.connector.getData(self.endpoint_company + path, params=params, headers=self.header)
+        self.listProjectIds = res
         if format == "raw":
             if save:
                 with open('projects.json', 'w') as f:
@@ -1368,9 +1282,11 @@ class Analytics:
             df.to_csv('projects.csv', index=False)
         return df
 
-    def getProject(self, projectId: str = None, projectClass: bool = False, retry: int = 0,verbose: bool = False) -> dict:
+    def getProject(self, projectId: str = None, projectClass: bool = False, retry: int = 0,verbose: bool = False) -> Union[dict,Project]:
         """
         Return the dictionary of the project information and its definition.
+        It will return a dictionary or a Project class.
+        The project detail will be saved as Project class in the projectsDetails class attribute.
         Arguments:
             projectId : REQUIRED : the project ID to be retrieved.
             projectClass : OPTIONAL : if set to True. Returns a class of the project with prefiltered information
@@ -1388,7 +1304,47 @@ class Analytics:
                 print('building an instance of Project class')
             myProject = Project(res)
             return myProject
+        self.projectsDetails[projectId] = Project(res)
         return res
+    
+    def getAllProjectDetails(self,projects:JsonListOrDataFrameType=None,filterNameProject:str=None,filterNameOwner:str=None,useAttribute:bool=True,verbose:bool=False)->dict:
+        """
+        Retrieve all projects details. You can either pass the list of dataframe returned from the getProjects methods and some filters.
+        Returns a dict of ProjectId and the value is the Project class for analysis.
+        Arguments:
+            projects : OPTIONAL : Takes the type of object returned from the getProjects (all data - not only the ID).
+                    If None is provided and you never ran the getProjects method, we will call the getProjects method and retrieve the elements.
+                    Otherwise you can pass either a limited list of elements that you want to check details for.
+            filterNameProject : OPTIONAL : If you want to retrieve project details for project with a specific string in their name.
+            filterNameOwner : OPTIONAL : If you want to retrieve project details for project with an owner having a specific name.
+            useAttribute : OPTIONAL : True by default, it will use the projectList saved in the listProjectIds attribute.
+                If you want to start from scratch on the retrieval process of your projects.
+        
+        Not using filter may end up taking a while to retrieve the information.
+        """
+        ## if no project data
+        if projects is None:
+            if len(self.listProjectIds)>0 and useAttribute:
+                fullProjectIds = self.listProjectIds
+            else:
+                fullProjectIds = self.getProjects(format='raw')
+        ## if project data is passed
+        elif projects is not None:
+            if isinstance(projects,pd.DataFrame):
+                fullProjectIds = projects.to_dict(orient='records')
+            elif isinstance(projects,list):
+                fullProjectIds = (proj['id'] for proj in projects)
+        if filterNameProject is not None:
+                fullProjectIds = [project for project in fullProjectIds if filterNameProject in project['name']]
+        if filterNameOwner is not None:
+                fullProjectIds = [project for project in fullProjectIds if filterNameOwner in project['owner'].get('name','')]
+        if verbose:
+            print(f'{len(fullProjectIds)} project details to retrieve')
+        projectIds = (project['id'] for project in fullProjectIds)
+        projectsDetails = {projectId:self.getProject(projectId,projectClass=True) for projectId in projectIds}
+        if filterNameProject is None and filterNameOwner is None:
+            self.projectsDetails = projectsDetails
+        return projectsDetails
 
     def deleteProject(self, projectId: str = None) -> dict:
         """
@@ -1459,6 +1415,174 @@ class Analytics:
         res = self.connector.postData(self.endpoint_company + path, data=projectObj, headers=self.header)
         return res
     
+    def findComponentsUsage(self,components:list=None,
+                            projectDetails:list=None,
+                            segments:Union[list,pd.DataFrame]=None,
+                            calculatedMetrics:Union[list,pd.DataFrame]=None,
+                            recursive:bool=False,
+                            regexUsed:bool=False,
+                            verbose:bool=False,
+                            resetProjectDetails:bool=False
+                            )->dict:
+        """
+        Find the usage of components in the different part of Adobe Analytics setup.
+        Projects, Segment, Calculated metrics.
+        Arguments:
+            components : REQUIRED : list of component to look for.
+                        Example : evar10,event1,prop3,segmentId, calculatedMetricsId
+            ProjectDetails: OPTIONAL : list of project details.
+            segments : OPTIONAL : If you wish to pass the segments to look for. (should contain definition)
+            calculatedMetrics : OPTIONAL : If you wish to pass the segments to look for. (should contain definition)
+            recursive : OPTIONAL : if set to True, will also find the reference where the meta component are used.
+                segments based on your elements will also be searched to see where they are located.
+            regexUsed : OPTIONAL : If set to True, the element are definied as a regex and some default setup is turned off.
+            resetProjectDetails : OPTIONAL : Set to false by default. If set to True, it will NOT use the cache.
+        """
+        listComponentProp = [comp for comp in components if 'prop' in comp]
+        listComponentVar = [comp for comp in components if 'evar' in comp]
+        listComponentEvent = [comp for comp in components if 'event' in comp]
+        listComponentSegs = [comp for comp in components if comp.startswith('s')]
+        listComponentCalcs = [comp for comp in components if comp.startswith('cm')]
+        restComponents = set(components) - set(listComponentProp+listComponentVar+listComponentEvent+listComponentSegs+listComponentCalcs)
+        listDefaultElements = [comp for comp in restComponents]
+        listRecusion = []
+        ## adding unrefular ones
+
+        regPartSeg = "('|\.)" ## ensure to not catch evar100 for evar10
+        regPartPro = "($|\.)" ## ensure to not catch evar100 for evar10
+        if regexUsed:
+            regPartSeg = ""
+            regPartPro = ""
+        if verbose:
+            print('retrieving segments')
+        if len(self.segments) == 0 and segments is None:
+            self.segments = self.getSegments(extended_info=True)
+            mySegments = self.segments
+        elif len(self.segments) > 0 and segments is None:
+            mySegments = self.segments
+        elif segments is not None:
+            if type(segments) == list:
+                mySegments = pd.DataFrame(segments)
+        else:
+            mySegments = segments
+        if verbose:
+            print('retrieving calculated metrics')
+        if len(self.calculatedMetrics) == 0 and calculatedMetrics is None:
+            self.calculatedMetrics = self.getCalculatedMetrics(extended_info=True)
+            myMetrics = self.calculatedMetrics
+        elif len(self.segments) > 0 and calculatedMetrics is None:
+            myMetrics = self.calculatedMetrics
+        elif calculatedMetrics is not None:
+            if type(calculatedMetrics) == list:
+                myMetrics = pd.DataFrame(calculatedMetrics)
+        else:
+            myMetrics = calculatedMetrics
+        if (len(self.projectsDetails) == 0 and projectDetails is None) or resetProjectDetails:
+            if verbose:
+                print('retrieving projects details - long process')
+            self.projectDetails = self.getAllProjectDetails(verbose=verbose)
+            myProjectDetails = (self.projectsDetails[key].to_dict() for key in self.projectsDetails)
+        elif len(self.projectsDetails) > 0 and projectDetails is None and resetProjectDetails==False:
+            if verbose:
+                print('transforming projects details')
+            myProjectDetails = (self.projectsDetails[key].to_dict() for key in self.projectsDetails)
+        elif projectDetails is not None:
+            if verbose:
+                print('setting the project details')
+            if isinstance(projectDetails[0],Project):
+                myProjectDetails = (item.to_dict() for item in projectDetails)
+            elif isinstance(projectDetails[0],dict):
+                myProjectDetails = (Project(item).to_dict() for item in projectDetails)
+        else:
+            raise Exception("Project details were not able to be processed")
+        returnObj = {element : {'segments':[],'calculatedMetrics':[],'projects':[]} for element in components}
+        recurseObj = defaultdict(list)
+        if verbose:
+            print('search started')
+            print(f'recursive option : {recursive}')
+        for _,seg in mySegments.iterrows():
+            for prop in listComponentProp:
+                if re.search(f"{prop+regPartSeg}",str(seg['definition'])):
+                    returnObj[prop]['segments'].append({seg['name']:seg['id']})
+                    if recursive:
+                        listRecusion.append(seg['id'])
+            for var in listComponentVar:
+                if re.search(f"{var+regPartSeg}",str(seg['definition'])):
+                    returnObj[var]['segments'].append({seg['name']:seg['id']})
+                    if recursive:
+                        listRecusion.append(seg['id'])
+            for event in listComponentEvent:
+                if re.search(f"{event}'",str(seg['definition'])):
+                    returnObj[event]['segments'].append({seg['name']:seg['id']})
+                    if recursive:
+                        listRecusion.append(seg['id'])
+            for element in listDefaultElements:
+                if re.search(f"{element}",str(seg['definition'])):
+                    returnObj[element]['segments'].append({seg['name']:seg['id']})
+                    if recursive:
+                        listRecusion.append(seg['id'])
+        for _,met in myMetrics.iterrows():
+            for prop in listComponentProp:
+                if re.search(f"{prop+regPartSeg}",str(met['definition'])):
+                    returnObj[prop]['calculatedMetrics'].append({met['name']:met['id']})
+                    if recursive:
+                        listRecusion.append(met['id'])
+            for var in listComponentVar:
+                if re.search(f"{var+regPartSeg}",str(met['definition'])):
+                    returnObj[var]['calculatedMetrics'].append({met['name']:met['id']})
+                    if recursive:
+                        listRecusion.append(met['id'])
+            for event in listComponentEvent:
+                if re.search(f"{event}'",str(met['definition'])):
+                    returnObj[event]['calculatedMetrics'].append({met['name']:met['id']})
+                    if recursive:
+                        listRecusion.append(met['id'])
+            for element in listDefaultElements:
+                if re.search(f"{element}'",str(met['definition'])):
+                    returnObj[element]['calculatedMetrics'].append({met['name']:met['id']})
+                    if recursive:
+                        listRecusion.append(met['id'])
+        for proj in myProjectDetails:
+            for prop in listComponentProp:
+                for element in proj['dimensions']:
+                    if re.search(f"{prop+regPartPro}",element):
+                        returnObj[prop]['projects'].append({proj['name']:proj['id']})
+            for var in listComponentVar:
+                for element in proj['dimensions']:
+                    if re.search(f"{var+regPartPro}",element):
+                        returnObj[var]['projects'].append({proj['name']:proj['id']})
+            for seg in listComponentSegs:
+                for element in proj['segments']:
+                    if re.search(f"{seg}",element):
+                        returnObj[seg]['projects'].append({proj['name']:proj['id']})
+            for met in listComponentCalcs:
+                for element in proj['calculatedMetrics']:
+                    if re.search(f"{met}",element):
+                        returnObj[met]['projects'].append({proj['name']:proj['id']})
+            for element in listDefaultElements:
+                for met in proj['calculatedMetrics']:
+                    if re.search(f"{element}",met):
+                        returnObj[element]['projects'].append({proj['name']:proj['id']})
+                for dim in proj['dimensions']:
+                    if re.search(f"{element}",dim):
+                        returnObj[element]['projects'].append({proj['name']:proj['id']})
+                for rsid in proj['rsids']:
+                    if re.search(f"{element}",rsid):
+                        returnObj[element]['projects'].append({proj['name']:proj['id']})
+            if recursive:
+                for rec in listRecusion:
+                    for element in proj['segments']:
+                        if re.search(f"{rec}",element):
+                            recurseObj[rec].append({proj['name']:proj['id']})
+                    for element in proj['calculatedMetrics']:
+                        if re.search(f"{rec}",element):
+                            recurseObj[rec].append({proj['name']:proj['id']})
+        if recursive:
+            returnObj['recursion'] = recurseObj
+        if verbose:
+            print('done')
+        return returnObj
+    
     def getUsageLogs(self,
         startDate:str=None,
         endDate:str=None,
