@@ -87,6 +87,12 @@ class AdobeRequest:
                 res = requests.get(endpoint, headers=headers, params=params, data=data)
             res_json = res.json()
         except:
+            ## handling 1.4
+            if kwargs.get('legacy',False):
+                try:
+                    return json.loads(res.text)
+                except:
+                    return res.text
             res_json = {'error': 'Request Error'}
             while internRetry > 0:
                 if kwargs.get("verbose", False):
@@ -118,12 +124,18 @@ class AdobeRequest:
             if res.status_code == 429 or res_json.get('error_code', None) == "429050":
                 res_json['status_code'] = 429
         except:
+            ## handling 1.4
+            if kwargs.get('legacy',False):
+                try:
+                    return json.loads(res.text)
+                except:
+                    return res.text
             res_json = {'error': 'Request Error'}
         return res_json
 
     def patchData(self, endpoint: str, params: dict = None, data=None, headers: dict = None, *args, **kwargs):
         """
-        Abstraction for deleting data
+        Abstraction for patching data
         """
         self._checkingDate()
         if headers is None:
@@ -142,7 +154,7 @@ class AdobeRequest:
 
     def putData(self, endpoint: str, params: dict = None, data=None, headers: dict = None, *args, **kwargs):
         """
-        Abstraction for deleting data
+        Abstraction for putting data
         """
         self._checkingDate()
         if headers is None:
